using System;
using System.Text;
using System.Collections;

namespace Indexer
{
	#region CarIsDeadException, take three.
	[System.Serializable]
	public class CarIsDeadException : System.ApplicationException
	{
		public CarIsDeadException() { }
		public CarIsDeadException(string message) : base( message ) { }
		public CarIsDeadException(string message, System.Exception inner) : base( message, inner ) { }
		public CarIsDeadException(
			System.Runtime.Serialization.SerializationInfo info,
			System.Runtime.Serialization.StreamingContext context) : base( info, context ) { }
	} 
    #endregion



	public class Radio
	{
		public void TurnOn(bool on)
		{
			if(on)
				Console.WriteLine("Jamming...");
			else
				Console.WriteLine("Quiet time...");
		}
	}


	public class Car
	{
		// Constant for maximum speed.
		public const int maxSpeed = 100;

		// Internal state data.
		private int currSpeed;
		private string petName;
		private int carID;

        #region Properties
		public int CurrSpeed
		{
			get
			{
				return currSpeed;
			}

			set
			{
				currSpeed = value;
			}
		}

		public string PetName
		{
			get
			{
				return petName;
			}

			set
			{
				petName = value;
			}
		} 
        #endregion

		// Is the car still operational?
		private bool carIsDead;

		// A car has-a radio.
		private Radio theMusicBox = new Radio();

		public Car() { }
		public Car(string name, int currSp)
		{
			currSpeed = currSp;
			petName = name;
		}
		public Car(string name, int currSp, int id)
		{
			currSpeed = currSp;
			petName = name;
			carID = id;
		}


		public void CrankTunes(bool state)
		{
			// Delegate request to inner object.
			theMusicBox.TurnOn(state);
		}

	}



	
	// Cars - is a container of Car objects. 
	public class Cars : IEnumerable, IEnumerator
	{ 
		public Car[] carArray; 

		//       
		int pos = -1; 

		//     Cars     Car 
		public Cars() 
		{ 
			carArray = new Car[10]; 
		}
 
		//   IEnumerator 
		public bool MoveNext() 
		{ 
			if(pos < carArray.Length) 
			{ 
				pos++; 
				return true; 
			} 
			else 
				return false; 
		} 

		public void Reset() 
		{ 
			pos = 0; 
		} 

		public object Current 
		{ 
			get 
			{ 
				return carArray[pos];
			} 
		} 
		//   IEnumerable 
		public IEnumerator GetEnumerator() 
		{ 
		{ 
			return (IEnumerator)this; 
		} 
		} 
		/// <summary>
		/// ////////////////////////////////////////////
		/// ///////////////////////////////////////////
		/// </summary>

		// The indexer returns a Car based on a numerical index. 
		public Car this[int pos] 
		{ 

			// Note ArrayList has an indexer as well! 
			get 
			{ 
				if(pos < 0 || pos > 10) 
    				throw new IndexOutOfRangeException("Out of range"); 
				else 
					return (Car)carArray[pos]; 
			} 

			set 
			{ 
				carArray.Add(value); 
			} 

		} 


	} 


	
	
	public class CarApp 
	{ 
		public static void Main(string[] args) 
		{ 
			Console.WriteLine("***** Fun with Indexers *****\n"); 

			// Assume the Garage type has an indexer method. 
			Cars carLot = new Cars(); 

			carLot.carArray[0] = new Car("FeeFee", 200); 
			carLot.carArray[1] = new Car("Clunker", 90); 
			carLot.carArray[2] = new Car("Zippy", 30); 

			// Now obtain and display each item. 
			for (int i = 0; i < 3; i++) 
			{ 
				Console.WriteLine("Car number: {0}", i); 
				Console.WriteLine("Name: {0}", carLot.carArray[i].PetName); 
				Console.WriteLine("Max speed: {0}", carLot.carArray[i].CurrSpeed); 
				Console.WriteLine(); 
			} 
			Console.ReadLine(); 
		} 
	} 

}
