using System;

namespace CarDelegate
{
	public class Car
	{
		// Define the Delegate types.
		public delegate void AboutToBlow(string msg);
		public delegate void Exploded (string msg);

		// Define member variables of each delegate. 
		private AboutToBlow almostDeadList;
		private Exploded explodedList;

		// Add member to the invocation list.
		public void OnAboutToBlow(AboutToBlow clientMethod)
		{ almostDeadList += clientMethod; }

		public void OnExploded(Exploded clientMethod)
		{ explodedList += clientMethod; }

		// Remove member to the invocation list.
		public void RemoveAboutToBlow(AboutToBlow clientMethod)
		{ almostDeadList -= clientMethod; }

		public void RemoveExploded(Exploded clientMethod)
		{ explodedList -= clientMethod; }

        #region Basic Car members...
        #region Nested radio
		// Radio as nested type
		public class Radio
		{
			public void TurnOn(bool on)
			{
				if (on)
					Console.WriteLine("Jamming...");
				else
					Console.WriteLine("Quiet time...");
			}
		}
        #endregion

		// Internal state data.
		private int currSpeed;
		private int maxSpeed;
		private string petName;

		// Is the car alive or dead?
		bool carIsDead;

		// A car has-a radio.
		private Radio theMusicBox = new Radio();

		public Car()
		{
			maxSpeed = 100;
		}

		public Car(string name, int max, int curr)
		{
			currSpeed = curr;
			maxSpeed = max;
			petName = name;
		}

		public void CrankTunes(bool state)
		{
			theMusicBox.TurnOn(state);
		}
#endregion 

		public void SpeedUp(int delta)
		{
			// If the car is dead, fire Exploded event.
			if (carIsDead)
			{
				if (explodedList != null)
					explodedList("Sorry, this car is dead...");
			}
			else
			{
				currSpeed += delta;

				// Almost dead?
				if (10 == maxSpeed - currSpeed
					&& almostDeadList != null)
				{
					almostDeadList("Careful buddy!  Gonna blow!");
				}

				// Still OK!
				if (currSpeed >= maxSpeed)
					carIsDead = true;
				else
					Console.WriteLine("->CurrSpeed = {0}", currSpeed);
			}
		}
	}

	
	class Class1
	{
			static void Main(string[] args)
			{
				Console.WriteLine("***** Delegates as event enablers *****");

				// Make a car as usual. 
				Car c1 = new Car("SlugBug", 100, 10);

				// Register event handlers.          
				Car.Exploded d = new Car.Exploded(CarExploded);
				c1.OnAboutToBlow(new Car.AboutToBlow(CarIsAlmostDoomed));
				c1.OnAboutToBlow(new Car.AboutToBlow(CarAboutToBlow));
				c1.OnExploded(d);

				// Speed up (this will generate the events.)
				Console.WriteLine("\n***** Speeding up *****");
				for (int i = 0; i < 6; i++)
					c1.SpeedUp(20);

				// Remove CarExploded method 
				// from invocation list.
				c1.RemoveExploded(d);

				Console.WriteLine("\n***** Speeding up *****");
				for (int i = 0; i < 6; i++)
					c1.SpeedUp(20);

				Console.ReadLine();
			}

		public static void CarAboutToBlow(string msg)
		{ Console.WriteLine(msg); }

		public static void CarIsAlmostDoomed(string msg)
		{ Console.WriteLine("Critical Message from Car: {0}", msg); }

		public static void CarExploded(string msg)
		{ Console.WriteLine(msg); }

	}
}
