using System;
//using System.Collections.Generic;
using System.Text;
using System.Collections;


namespace CarDelegate
{
	public class Car
	{
		// ! 
		private bool isDirty; //   ? 
		private bool shouldRotate; //   ? 

		//  isDirty 
		public bool Dirty 
		{ 
			get {return isDirty;}
			set {isDirty = value;} 
		} 
		//  shouldRotate 
		public bool Rotate 
		{ 
			get {return shouldRotate;} 
			set {shouldRotate = value;} 
		} 


		#region Basic Car members...
		#region Nested radio
		// Radio as nested type
		public class Radio
		{
			public void TurnOn(bool on)
			{
				if (on)
					Console.WriteLine("Jamming...");
				else
					Console.WriteLine("Quiet time...");
			}
		}
        #endregion

		// Internal state data.
		private int currSpeed;
		private int maxSpeed;
		private string petName;

		// Is the car alive or dead?
//		bool carIsDead;

		// A car has-a radio.
		private Radio theMusicBox = new Radio();

		public Car()
		{
			maxSpeed = 100;
		}

		public Car(string name, int max, int curr)
		{
			currSpeed = curr;
			maxSpeed = max;
			petName = name;
		}

		//    
		public Car(string name, int max, int curr, bool dirty, bool rotate) 
		{ 
			currSpeed = curr;
			maxSpeed = max;
			petName = name;
			isDirty = dirty; 
			shouldRotate = rotate; 
		} 

		
		public void CrankTunes(bool state)
		{
			theMusicBox.TurnOn(state);
		}

		
		#endregion 

		// -  ,    .   
		//    - ,    
		//   Car    : 

		public delegate void CarDelegate(Car c); 
	}

	public class Garage 
	{ 

		// We have some cars. 
		ArrayList theCars = new ArrayList(); 

		public Garage() 
		{ 

			theCars.Add(new Car("Viper", 100, 0, true, false)); 
			theCars.Add(new Car("Fred", 100, 0, false, false)); 
			theCars.Add(new Car("BillyBob", 100, 0, false, true)); 
			theCars.Add(new Car("Bart", 100, 0, true, true)); 
			theCars.Add(new Car("Stan", 100, 0, false, true)); 

		} 


		// This method takes a CarDelegate as a parameter. 
		// Therefore! 'proc' is nothing more than a function pointer... 

		public void ProcessCars(Car.CarDelegate proc) 
		{ 
			//   ? 
			foreach(Delegate d in proc.GetInvocationList()) 
			{ 
				Console.WriteLine("***** Calling " + d.Method.ToString() + " *****\n"); 

			} 

			// Am I calling an object's method or a static method? 
			if (proc.Target != null) 
				Console.WriteLine("\n-->Target: {0}", proc.Target); 

			else 
				Console.WriteLine("\n-->Target is a static method"); 


			// Now call method for each car. 
			foreach (Car c in theCars) 
			{ 
				Console.WriteLine("\n-> Processing a Car"); 
				proc(c); 
			} 
			Console.WriteLine(); 
		} 
	} 


	
	class CarApp
	{
		public static void WashCar(Car c) 
		{ 
			if(c.Dirty) 
				Console.WriteLine("Cleaning a car"); 
			else 
				Console.WriteLine("This car is already clean ..."); 
		} 

		public static void RotateTires(Car c) 
		{ 
			if(c.Rotate) 
				Console.WriteLine("Tires have been rotated"); 
			else 
				Console.WriteLine(" Dont need to be rotated ..."); 
		} 

		static int Main(string[] args) 
		{ 
			// Make the garage. 
			Garage g = new Garage(); 

			//     
			Car.CarDelegate wash = new Car.CarDelegate(WashCar); 
			Car.CarDelegate rotate = new Car.CarDelegate(RotateTires); 

			//    ,       
			MulticastDelegate d = wash + rotate; 

			//    ProcessCars() 
			g.ProcessCars((Car.CarDelegate)d); 

			//  Remove()    -    
			//     
			Delegate washOnly = MulticastDelegate.Remove(d, rotate); 
			g.ProcessCars((Car.CarDelegate)washOnly); 

			Console.ReadLine(); 

			return 0; 
		} 
	}
}
