using System;
using System.Text;
using System.Collections;

namespace CarEvents
{
	public class Car
	{
		// This delegate works in conjunction with the
		// Cars events.
		public delegate void CarEventHandler(string msg);

		// This car can send these events.
		public event CarEventHandler Exploded;
		public event CarEventHandler AboutToBlow;

        #region Basic Car members...
        #region Nested radio
		// Radio as nested type
		public class Radio
		{
			public void TurnOn(bool on)
			{
				if (on)
					Console.WriteLine("Jamming...");
				else
					Console.WriteLine("Quiet time...");
			}
		}
        #endregion

		// Internal state data.
		private int currSpeed;
		private int maxSpeed;
		private string petName;

		// Is the car alive or dead?
		bool carIsDead;

		// A car has-a radio.
		private Radio theMusicBox = new Radio();

		public Car()
		{
			maxSpeed = 100;
		}

		public Car(string name, int max, int curr)
		{
			currSpeed = curr;
			maxSpeed = max;
			petName = name;
		}

		public void CrankTunes(bool state)
		{
			theMusicBox.TurnOn(state);
		}
#endregion 

		public void SpeedUp(int delta)
		{
			// If the car is dead, fire Exploded event.
			if (carIsDead)
			{
				if (Exploded != null)
					Exploded("Sorry, this car is dead...");
			}

			else
			{
				currSpeed += delta;

				// Almost dead?
				if (10 == maxSpeed - currSpeed
					&& AboutToBlow != null)
				{
					AboutToBlow("Careful buddy!  Gonna blow!");
				}

				// Still OK!
				if (currSpeed >= maxSpeed)
					carIsDead = true;
				else
					Console.WriteLine("->CurrSpeed = {0}", currSpeed);
			}
		}
	}


	class Class1
	{
			static void Main(string[] args)
			{
				Console.WriteLine("***** Events *****");

				// Make a car as usual. 
				Car c1 = new Car("SlugBug", 100, 10);

				// Register event handlers.          
				c1.AboutToBlow += new Car.CarEventHandler(CarIsAlmostDoomed);
				c1.AboutToBlow += new Car.CarEventHandler(CarAboutToBlow);

				Car.CarEventHandler d = new Car.CarEventHandler(CarExploded); 
				c1.Exploded += d;
            
				// Speed up (this will generate the events.)
				Console.WriteLine("\n***** Speeding up *****");
				for (int i = 0; i < 6; i++)
					c1.SpeedUp(20);

				// Remove CarExploded method 
				// from invocation list.
				c1.Exploded -= d;

				Console.WriteLine("\n***** Speeding up *****");
				for (int i = 0; i < 6; i++)
					c1.SpeedUp(20);

				Console.ReadLine();
			}

		public static void CarAboutToBlow(string msg)
		{ Console.WriteLine(msg); }

		public static void CarIsAlmostDoomed(string msg)
		{ Console.WriteLine("Critical Message from Car: {0}", msg); }

		public static void CarExploded(string msg)
		{ Console.WriteLine(msg); }

	}
}
