using System;
using System.Text;
using System.Collections;

namespace CarEvents
{
	public class Car
	{
		// This delegate works in conjunction with the
		// Cars events.
		public delegate void CarEventHandler(string msg);

		// This car can send these events.
//		public event CarEventHandler Exploded;
//		public event CarEventHandler AboutToBlow;

        #region Basic Car members...
        #region Nested radio
		// Radio as nested type
		public class Radio
		{
			public void TurnOn(bool on)
			{
				if (on)
					Console.WriteLine("Jamming...");
				else
					Console.WriteLine("Quiet time...");
			}
		}
        #endregion

		// Internal state data.
		private int currSpeed;
		private int maxSpeed;
		private string petName;

		// Is the car alive or dead?
//		bool carIsDead;

		//       
		private bool dead; 

		//. ,       
		public delegate void EngineHandler(string msg); 

		//  
		public static event EngineHandler Exploded; 
		public static event EngineHandler AboutToBlow; 



		// A car has-a radio.
		private Radio theMusicBox = new Radio();

		public Car()
		{
			maxSpeed = 100;
		}

		public Car(string name, int max, int curr)
		{
			currSpeed = curr;
			maxSpeed = max;
			petName = name;
		}

		public void CrankTunes(bool state)
		{
			theMusicBox.TurnOn(state);
		}
#endregion 

	
		public void SpeedUp(int delta) 
		{ 

			if(dead) 
			{ 

				if(Exploded != null) 
					Exploded("Sorry, this car is dead"); 

			} 
			else 
			{ 
				currSpeed += delta; 

				if(10 == maxSpeed - currSpeed) 
					if(AboutToBlow != null) 
						AboutToBlow("Carefool, approaching terminal speed"); 


				if(currSpeed >= maxSpeed) 
					dead = true; 
				else 
					Console.WriteLine("\t CurrSpeed = {0}", currSpeed); 


			} 

		} 

	}


	class CarApp
	{
		static int Main(string[] args)
		{
			Console.WriteLine("***** Events *****");

			// Make a car as usual. 
			Car c1 = new Car("SlugBug", 100, 10);

			// Register event handlers.          
			Car.Exploded += new Car.EngineHandler(OnBlowUp);
			Car.AboutToBlow += new Car.EngineHandler(OnAboutToBlow);

			// Speed up (this will generate the events.)
			Console.WriteLine("\n***** Speeding up *****");
			for (int i = 0; i < 6; i++)
				c1.SpeedUp(20);

			// Register event handlers.          
			Car.Exploded -= new Car.EngineHandler(OnBlowUp);
			Car.AboutToBlow -= new Car.EngineHandler(OnAboutToBlow);

			Console.WriteLine("\n***** Speeding up *****");
			for (int i = 0; i < 10; i++)
				c1.SpeedUp(20);

			Console.ReadLine();
			return 0;
		}

		public static void OnBlowUp(string s)
		{ 
			Console.WriteLine("Message from car: {0}", s); 
		}

		public static void OnAboutToBlow(string s)
		{ 
			Console.WriteLine("Message from car: {0}", s); 
		}

	}
}
