#region Using directives

using System;
//using System.Collections.Generic;
using System.Text;

#endregion

using System.Collections;

namespace EventInterface
{
    // The callback interface.
public interface IEngineStatus
{
void AboutToBlow(string msg);
void Exploded(string msg);
}

public class Car
{
        // The set of connected sinks.
ArrayList clientSinks = new ArrayList();

        // Attach or disconnect from the source of events.
public void Advise(IEngineStatus sink)
{ clientSinks.Add(sink); }

public void Unadvise(IEngineStatus sink)
{ clientSinks.Remove(sink); }

        #region Basic Car members...

        // Internal state data.
private int currSpeed;
private int maxSpeed;
private string petName;

        // Is the car alive or dead?
bool carIsDead;


public Car()
{
maxSpeed = 100;
}

public Car(string name, int max, int curr)
{
currSpeed = curr;
maxSpeed = max;
petName = name;
}

#endregion 

public void Accelerate(int delta)
{
            // If the car is dead, send Exploded event.
if (carIsDead)
{
foreach (IEngineStatus e in clientSinks)
e.Exploded("Sorry, this car is dead...");
}
else
{
currSpeed += delta;

                // Almost dead?
if (10 == maxSpeed - currSpeed)
{
foreach (IEngineStatus e in clientSinks)
e.AboutToBlow("Careful buddy!  Gonna blow!");
}

                // Still OK!
if (currSpeed >= maxSpeed)
carIsDead = true;
else
Console.WriteLine("->CurrSpeed = " + currSpeed);
}
}
}
    #region The Sink object
	public class CarEventSink : IEngineStatus
	{
		private string name;
		public CarEventSink() { }
		public CarEventSink(string sinkName)
		{ name = sinkName; }
		public void AboutToBlow(string msg)
		{ Console.WriteLine("{0} reporting: {1}", name, msg); }
		public void Exploded(string msg)
		{ Console.WriteLine("{0} reporting: {1}", name, msg); }
	}
    #endregion

	class Program
	{
		static void Main(string[] args)
		{
			Console.WriteLine("***** Interfaces as callbacks *****");

			// Make a car as usual. 
			Car c1 = new Car("SlugBug", 100, 10);

			// Make sink object.
			Console.WriteLine("\n***** Creating sinks *****");
			CarEventSink sink = new CarEventSink("First sink");
			CarEventSink myOtherSink = new CarEventSink("Other sink");

			// Hand sink to Car.
			Console.WriteLine("\n***** Sending 2 sinks into Car *****");
			c1.Advise(sink);
			c1.Advise(myOtherSink);

			// Speed up (this will generate the notifications.)
			Console.WriteLine("\n***** Speeding up *****");
			for (int i = 0; i < 6; i++)
				c1.Accelerate(20);

			// Detach sink from events.
			Console.WriteLine("\n***** Removing first sink *****");
			c1.Unadvise(sink);

			// Speed up again (only myothersink will be called.)
			Console.WriteLine("\n***** Speeding up again *****");
			for (int i = 0; i < 6; i++)
				c1.Accelerate(20);

			// Detach other sink from events.
			Console.WriteLine("\n***** Removing second sink *****");
			c1.Unadvise(myOtherSink);

			Console.ReadLine();
		}
	}
}
